@@ -146,11 +146,11 @@ class ValueNumberer(object):
         num_gd = len(mt.global_derivatives)
         assert not (num_ld and num_gd)
         if num_ld:
-            domain = mt.terminal.ufl_domain()
+            domain = ufl.domain.extract_unique_domain(mt.terminal)
             tdim = domain.topological_dimension()
             d_components = ufl.permutation.compute_indices((tdim, ) * num_ld)
         elif num_gd:
-            domain = mt.terminal.ufl_domain()
+            domain = ufl.domain.extract_unique_domiain(mt.terminal)
             gdim = domain.geometric_dimension()
             d_components = ufl.permutation.compute_indices((gdim, ) * num_gd)
         else:
