@@ -16,10 +16,9 @@ import re
 import string
 
 import ufl
-
 from ffcx import __version__ as FFCX_VERSION
 from ffcx import compiler, formatting
-from ffcx.parameters import FFCX_DEFAULT_PARAMETERS, get_parameters
+from ffcx.options import FFCX_DEFAULT_OPTIONS, get_options
 
 logger = logging.getLogger("ffcx")
 
@@ -31,10 +30,10 @@ parser.add_argument("-o", "--output-directory", type=str, default=".", help="out
 parser.add_argument("--visualise", action="store_true", help="visualise the IR graph")
 parser.add_argument("-p", "--profile", action='store_true', help="enable profiling")
 
-# Add all parameters from FFCx parameter system
-for param_name, (param_val, param_desc) in FFCX_DEFAULT_PARAMETERS.items():
-    parser.add_argument(f"--{param_name}",
-                        type=type(param_val), help=f"{param_desc} (default={param_val})")
+# Add all options from FFCx option system
+for opt_name, (opt_val, opt_desc) in FFCX_DEFAULT_OPTIONS.items():
+    parser.add_argument(f"--{opt_name}",
+                        type=type(opt_val), help=f"{opt_desc} (default={opt_val})")
 
 parser.add_argument("ufl_file", nargs='+', help="UFL file(s) to be compiled")
 
@@ -42,9 +41,9 @@ parser.add_argument("ufl_file", nargs='+', help="UFL file(s) to be compiled")
 def main(args=None):
     xargs = parser.parse_args(args)
 
-    # Parse all other parameters
-    priority_parameters = {k: v for k, v in xargs.__dict__.items() if v is not None}
-    parameters = get_parameters(priority_parameters)
+    # Parse all other options
+    priority_options = {k: v for k, v in xargs.__dict__.items() if v is not None}
+    options = get_options(priority_options)
 
     # Call parser and compiler for each file
     for filename in xargs.ufl_file:
@@ -67,7 +66,7 @@ def main(args=None):
         # Generate code
         code_h, code_c = compiler.compile_ufl_objects(
             ufd.forms + ufd.expressions + ufd.elements, ufd.object_names,
-            prefix=prefix, parameters=parameters, visualise=xargs.visualise)
+            prefix=prefix, options=options, visualise=xargs.visualise)
 
         # Write to file
         formatting.write_code(code_h, code_c, prefix, xargs.output_directory)
